<?php
/**
 * PGA Payment Gateway.
 *
 * Provides a PGA Payment Gateway.
 *
 * @class        WC_Gateway_PGA
 * @extends      WC_Payment_Gateway
 * @version      1.0.0
 */

if ( ! defined( 'ABSPATH' ) )
{
	exit;
}

/**
 * WC_Gateway_PGA Class.
 */
class WC_Gateway_PGA extends WC_Payment_Gateway
{
	/** @var bool Whether or not logging is enabled */
	public static $log_enabled = false;

	/** @var WC_Logger Logger instance */
	public static $log = false;

	protected $order = null;

	protected $debug = false;

	protected $merchant_id;
	protected $account_id;

	/**
	 * Constructor for the gateway.
	 */
	public function __construct()
	{
		$this->id                 = 'pga';
		$this->has_fields         = true;
		$this->order_button_text  = __( 'Place order', 'woocommerce' );
		$this->method_title       = __( 'Платежный шлюз PGA', 'woocommerce' );
		$this->method_description = 'Прием платежей по пластиковым картам Visa & MasterCard';

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables.
		$this->title       = $this->get_option( 'title' );
		$this->description = $this->get_option( 'description' );

		$this->enabled     = $this->get_option( 'enabled', 'no' );
		$this->debug       = 'yes' === $this->get_option( 'debug', 'no' );

		$this->merchant_id = $this->get_option( 'merchant_id' );
		$this->account_id  = $this->get_option( 'account_id' );

		self::$log_enabled = $this->debug;

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
			$this,
			'process_admin_options'
		) );
	}

	public function init_form_fields()
	{
		$this->form_fields = array(
			'enabled'      => array(
				'title'   => __( 'Вкл/Выкл', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Включить PGA Gateway', 'woocommerce' ),
				'default' => 'yes'
			),
			'debug'        => array(
				'title'       => __( 'Логирование', 'woocommerce' ),
				'type'        => 'checkbox',
				'label'       => __( 'Активировать логирование', 'woocommerce' ),
				'default'     => 'no',
				'description' => sprintf( __( 'Записывать события в лог <code>%s</code>', 'woocommerce' ), wc_get_log_file_path( 'pga' ) )
			),
			'title'        => array(
				'title'       => __( 'Название', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Название метода платежа, которое пользователь видит при оформлении заказа', 'woocommerce' ),
				'default'     => __( 'Оплата пластиковой картой', 'woocommerce' )
			),
			'description' => array(
				'title'       => __( 'Описание', 'woocommerce' ),
				'type'        => 'text',
				//'desc_tip'    => true,
				'description' => __( 'Этот текст пользователь видит на странице оформления заказа при выборе метода платежа', 'woocommerce' ),
				'default'     => __( 'Оплата пластиковой картой Visa или MasterCard', 'woocommerce' )
			),

			'advanced' => array(
				'title'       => __( 'Настройка магазина', 'woocommerce' ),
				'type'        => 'title',
				'description' => '',
			),
			'merchant_id' => array(
				'title'       => __( 'PCID Магазина', 'woocommerce' ),
				'type'        => 'text',
				'default'     => __( '', 'woocommerce' )
			),
			'account_id'  => array(
				'title'       => __( 'PCID Аккаунта', 'woocommerce' ),
				'type'        => 'text',
				'default'     => __( '', 'woocommerce' )
			),
			'start_payment_url'  => array(
				'title'       => __( 'Адрес запроса', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Адрес запроса для инициации платежа', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'cert_path'  => array(
				'title'       => __( 'Путь к сертификату', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Укажите путь до сертификата от корня сайта. Например: "/path/to/cert/certificate_file.crt"', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'currency'  => array(
				'title'       => __( 'Код валюты', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Код валюты по ISO 4217 (Рубль = 643)', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'success_url'  => array(
				'title'       => __( 'Адрес при успешной оплате', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Если не заполнять, будет использоваться стандартная страница WooCommerce', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'decline_url'  => array(
				'title'       => __( 'Адрес при ошибке оплаты', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'URL (на веб-сайте продавца) для перенаправления плательщика при неуспешном платеже', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'lang_code'  => array(
				'title'       => __( 'Язык диалога с пользователем', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Двухсимвольный код в соответствии со стандартом ISO 639. Если параметр не указан, используется язык по умолчанию.', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
			'page_id'  => array(
				'title'       => __( 'Идентификатор платежной страницы', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Строка длиной 32 символа. Если параметр не указан, используется платежная страница по умолчанию.', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' )
			),
		);
	}

	public function process_payment($order_id)
	{
		$this->order = new PgaOrder($order_id);

		$getParam['o.order_id'] = $order_id;
        $getParam['o.plugin'] = "gcs_wp_wo_0.1";
        $getParam['merch_id'] = $this->settings["merchant_id"];

		$getParam['back_url_s'] = ($this->settings["success_url"]!="")
			? $this->settings["success_url"]
			: $this->get_return_url($this->order); // Стандартная страница WC: "Thank you. Your order has been received."

        $getParam['back_url_f'] = $this->settings["decline_url"];

		if (!empty($this->settings[ 'lang_code' ])) {
			$getParam['lang'] = $this->settings['lang_code'];
		}

		if (!empty($this->settings['page_id'])) {
			$getParam['page_id'] = $this->settings['page_id'];
		}

		$args = http_build_query($getParam, null, '&');

		$this->order->add_order_note( 'PGA: пользователь переходит на платежный шлюз для ввода данных пластиковой карты' );

		return array(
			'result' => 'success',
			'redirect' => $this->settings['start_payment_url']. '?' . $args
		);
	}

	/**
	 * Logging method.
	 *
	 * @param string $message
	 */
	public static function log( $message )
	{
		if ( self::$log_enabled )
		{
			if ( empty( self::$log ) )
			{
				self::$log = new WC_Logger();
			}
			self::$log->add( 'pga', $message );
		}
	}
}